@@ -51,11 +51,7 @@ PERFORMANCE OF THIS SOFTWARE.
 #include "errmsg.h"
 
 #define MyAlloc(s,t) (s *) t.tp_alloc(&t,0)
-#ifdef IS_PY3K
-# define MyFree(o) Py_TYPE(o)->tp_free((PyObject*)o)
-#else
-# define MyFree(ob) ob->ob_type->tp_free((PyObject *)ob)
-#endif
+#define MyFree(o) Py_TYPE(o)->tp_free((PyObject*)o)
 
 static PyObject *_mysql_MySQLError;
 static PyObject *_mysql_Warning;
@@ -95,11 +91,7 @@ typedef struct {
 extern PyTypeObject _mysql_ResultObject_Type;
 
 static int _mysql_server_init_done = 0;
-#if MYSQL_VERSION_ID >= 40000
 #define check_server_init(x) if (!_mysql_server_init_done) { if (mysql_server_init(0, NULL, NULL)) { _mysql_Exception(NULL); return x; } else { _mysql_server_init_done = 1;} }
-#else
-#define check_server_init(x) if (!_mysql_server_init_done) _mysql_server_init_done = 1
-#endif
 
 #if MYSQL_VERSION_ID >= 50500
 #define HAVE_OPENSSL 1
@@ -255,7 +247,6 @@ static PyObject *_mysql_server_init(
 					 &cmd_args, &groups))
 		return NULL;
 
-#if MYSQL_VERSION_ID >= 40000
 	if (cmd_args) {
 		if (!PySequence_Check(cmd_args)) {
 			PyErr_SetString(PyExc_TypeError,
@@ -322,7 +313,6 @@ static PyObject *_mysql_server_init(
 		_mysql_Exception(NULL);
 		goto finish;
 	}
-#endif
 	ret = Py_None;
 	Py_INCREF(Py_None);
 	_mysql_server_init_done = 1;
@@ -340,9 +330,7 @@ static PyObject *_mysql_server_end(
 	PyObject *self,
 	PyObject *args) {
 	if (_mysql_server_init_done) {
-#if MYSQL_VERSION_ID >= 40000
 		mysql_server_end();
-#endif
 		_mysql_server_init_done = 0;
 		Py_INCREF(Py_None);
 		return Py_None;
@@ -350,20 +338,18 @@ static PyObject *_mysql_server_end(
 	return _mysql_Exception(NULL);
 }
 
-#if MYSQL_VERSION_ID >= 32314
 static char _mysql_thread_safe__doc__[] =
 "Indicates whether the client is compiled as thread-safe.";
 
 static PyObject *_mysql_thread_safe(
 	PyObject *self,
-	PyObject *args) {
+	PyObject *noargs)
+{
 	PyObject *flag;
-	if (!PyArg_ParseTuple(args, "")) return NULL;
 	check_server_init(NULL);
 	if (!(flag=PyInt_FromLong((long)mysql_thread_safe()))) return NULL;
 	return flag;
 }
-#endif
 
 static char _mysql_ResultObject__doc__[] =
 "result(connection, use=0, converter={}) -- Result set from a query.\n\
@@ -388,8 +374,8 @@ _mysql_ResultObject_Initialize(
 	int n, i;
 	MYSQL_FIELD *fields;
 
-	if (!PyArg_ParseTupleAndKeywords(args, kwargs, "O|iO", kwlist,
-					  &conn, &use, &conv))
+	if (!PyArg_ParseTupleAndKeywords(args, kwargs, "O!|iO", kwlist,
+					 &_mysql_ConnectionObject_Type, &conn, &use, &conv))
 		return -1;
 	if (!conv) {
 		if (!(conv = PyDict_New()))
@@ -530,10 +516,12 @@ _mysql_ConnectionObject_Initialize(
 	PyObject *ssl = NULL;
 #if HAVE_OPENSSL
 	char *key = NULL, *cert = NULL, *ca = NULL,
-		*capath = NULL, *cipher = NULL;
+		 *capath = NULL, *cipher = NULL;
+	PyObject *ssl_keepref[5] = {NULL};
+	int n_ssl_keepref = 0;
 #endif
 	char *host = NULL, *user = NULL, *passwd = NULL,
-		*db = NULL, *unix_socket = NULL;
+		 *db = NULL, *unix_socket = NULL;
 	unsigned int port = 0;
 	unsigned int client_flag = 0;
 	static char *kwlist[] = { "host", "user", "passwd", "db", "port",
@@ -586,11 +574,11 @@ _mysql_ConnectionObject_Initialize(
 
 #ifdef IS_PY3K
 #define _stringsuck(d,t,s) {t=PyMapping_GetItemString(s,#d);\
-        if(t){d=PyUnicode_AsUTF8(t);Py_DECREF(t);}\
+        if(t){d=PyUnicode_AsUTF8(t);ssl_keepref[n_ssl_keepref++]=t;}\
         PyErr_Clear();}
 #else
 #define _stringsuck(d,t,s) {t=PyMapping_GetItemString(s,#d);\
-        if(t){d=PyString_AsString(t);Py_DECREF(t);}\
+        if(t){d=PyString_AsString(t);ssl_keepref[n_ssl_keepref++]=t;}\
         PyErr_Clear();}
 #endif
 
@@ -645,9 +633,9 @@ _mysql_ConnectionObject_Initialize(
 		mysql_options(&(self->connection), MYSQL_OPT_LOCAL_INFILE, (char *) &local_infile);
 
 #if HAVE_OPENSSL
-	if (ssl)
-		mysql_ssl_set(&(self->connection),
-			      key, cert, ca, capath, cipher);
+	if (ssl) {
+		mysql_ssl_set(&(self->connection), key, cert, ca, capath, cipher);
+	}
 #endif
 
 	conn = mysql_real_connect(&(self->connection), host, user, passwd, db,
@@ -655,12 +643,22 @@ _mysql_ConnectionObject_Initialize(
 
 	Py_END_ALLOW_THREADS ;
 
+#if HAVE_OPENSSL
+	if (ssl) {
+		int i;
+		for (i=0; i<n_ssl_keepref; i++) {
+			Py_DECREF(ssl_keepref[i]);
+			ssl_keepref[i] = NULL;
+		}
+	}
+#endif
+
 	if (!conn) {
 		_mysql_Exception(self);
 		return -1;
 	}
 
-        /* Internal references to python-land objects */
+	/* Internal references to python-land objects */
 	if (!conv)
 		conv = PyDict_New();
 	else
@@ -785,11 +783,8 @@ static char _mysql_ConnectionObject_close__doc__[] =
 static PyObject *
 _mysql_ConnectionObject_close(
 	_mysql_ConnectionObject *self,
-	PyObject *args)
+	PyObject *noargs)
 {
-	if (args) {
-		if (!PyArg_ParseTuple(args, "")) return NULL;
-	}
 	if (self->open) {
 		Py_BEGIN_ALLOW_THREADS
 		mysql_close(&(self->connection));
@@ -813,11 +808,14 @@ Non-standard. Use Cursor.rowcount.\n\
 static PyObject *
 _mysql_ConnectionObject_affected_rows(
 	_mysql_ConnectionObject *self,
-	PyObject *args)
+	PyObject *noargs)
 {
-	if (!PyArg_ParseTuple(args, "")) return NULL;
+	my_ulonglong ret;
 	check_connection(self);
-	return PyLong_FromUnsignedLongLong(mysql_affected_rows(&(self->connection)));
+	ret = mysql_affected_rows(&(self->connection));
+	if (ret == (my_ulonglong)-1)
+		return PyInt_FromLong(-1);
+	return PyLong_FromUnsignedLongLong(ret);
 }
 
 static char _mysql_debug__doc__[] =
@@ -847,10 +845,9 @@ this to work. Non-standard.\n\
 static PyObject *
 _mysql_ConnectionObject_dump_debug_info(
 	_mysql_ConnectionObject *self,
-	PyObject *args)
+	PyObject *noargs)
 {
 	int err;
-	if (!PyArg_ParseTuple(args, "")) return NULL;
 	check_connection(self);
 	Py_BEGIN_ALLOW_THREADS
 	err = mysql_dump_debug_info(&(self->connection));
@@ -871,15 +868,7 @@ _mysql_ConnectionObject_autocommit(
 	int flag, err;
 	if (!PyArg_ParseTuple(args, "i", &flag)) return NULL;
 	Py_BEGIN_ALLOW_THREADS
-#if MYSQL_VERSION_ID >= 40100
 	err = mysql_autocommit(&(self->connection), flag);
-#else
-	{
-		char query[256];
-		snprintf(query, 256, "SET AUTOCOMMIT=%d", flag);
-		err = mysql_query(&(self->connection), query);
-	}
-#endif
 	Py_END_ALLOW_THREADS
 	if (err) return _mysql_Exception(self);
 	Py_INCREF(Py_None);
@@ -906,16 +895,11 @@ static char _mysql_ConnectionObject_commit__doc__[] =
 static PyObject *
 _mysql_ConnectionObject_commit(
 	_mysql_ConnectionObject *self,
-	PyObject *args)
+	PyObject *noargs)
 {
 	int err;
-	if (!PyArg_ParseTuple(args, "")) return NULL;
 	Py_BEGIN_ALLOW_THREADS
-#if MYSQL_VERSION_ID >= 40100
 	err = mysql_commit(&(self->connection));
-#else
-	err = mysql_query(&(self->connection), "COMMIT");
-#endif
 	Py_END_ALLOW_THREADS
 	if (err) return _mysql_Exception(self);
 	Py_INCREF(Py_None);
@@ -928,16 +912,11 @@ static char _mysql_ConnectionObject_rollback__doc__[] =
 static PyObject *
 _mysql_ConnectionObject_rollback(
 	_mysql_ConnectionObject *self,
-	PyObject *args)
+	PyObject *noargs)
 {
 	int err;
-	if (!PyArg_ParseTuple(args, "")) return NULL;
 	Py_BEGIN_ALLOW_THREADS
-#if MYSQL_VERSION_ID >= 40100
 	err = mysql_rollback(&(self->connection));
-#else
-	err = mysql_query(&(self->connection), "ROLLBACK");
-#endif
 	Py_END_ALLOW_THREADS
 	if (err) return _mysql_Exception(self);
 	Py_INCREF(Py_None);
@@ -960,22 +939,16 @@ Non-standard.\n\
 static PyObject *
 _mysql_ConnectionObject_next_result(
 	_mysql_ConnectionObject *self,
-	PyObject *args)
+	PyObject *noargs)
 {
 	int err;
-	if (!PyArg_ParseTuple(args, "")) return NULL;
 	Py_BEGIN_ALLOW_THREADS
-#if MYSQL_VERSION_ID >= 40100
 	err = mysql_next_result(&(self->connection));
-#else
-	err = -1;
-#endif
 	Py_END_ALLOW_THREADS
 	if (err > 0) return _mysql_Exception(self);
 	return PyInt_FromLong(err);
 }		
 
-#if MYSQL_VERSION_ID >= 40100
 
 static char _mysql_ConnectionObject_set_server_option__doc__[] =
 "set_server_option(option) -- Enables or disables an option\n\
@@ -1013,9 +986,8 @@ Non-standard.\n\
 static PyObject *
 _mysql_ConnectionObject_sqlstate(
 	_mysql_ConnectionObject *self,
-	PyObject *args)
+	PyObject *noargs)
 {
-	if (!PyArg_ParseTuple(args, "")) return NULL;
 	return PyString_FromString(mysql_sqlstate(&(self->connection)));
 }
 
@@ -1028,14 +1000,11 @@ Non-standard.\n\
 static PyObject *
 _mysql_ConnectionObject_warning_count(
 	_mysql_ConnectionObject *self,
-	PyObject *args)
+	PyObject *noargs)
 {
-	if (!PyArg_ParseTuple(args, "")) return NULL;
 	return PyInt_FromLong(mysql_warning_count(&(self->connection)));
 }		
 
-#endif
-
 static char _mysql_ConnectionObject_errno__doc__[] =
 "Returns the error code for the most recently invoked API function\n\
 that can succeed or fail. A return value of zero means that no error\n\
@@ -1045,9 +1014,8 @@ occurred.\n\
 static PyObject *
 _mysql_ConnectionObject_errno(
 	_mysql_ConnectionObject *self,
-	PyObject *args)
+	PyObject *noargs)
 {
-	if (!PyArg_ParseTuple(args, "")) return NULL;
 	check_connection(self);
 	return PyInt_FromLong((long)mysql_errno(&(self->connection)));
 }
@@ -1061,9 +1029,8 @@ occurred.\n\
 static PyObject *
 _mysql_ConnectionObject_error(
 	_mysql_ConnectionObject *self,
-	PyObject *args)
+	PyObject *noargs)
 {
-	if (!PyArg_ParseTuple(args, "")) return NULL;
 	check_connection(self);
 	return PyString_FromString(mysql_error(&(self->connection)));
 }
@@ -1088,18 +1055,19 @@ _mysql_escape_string(
 	str = PyBytes_FromStringAndSize((char *) NULL, size*2+1);
 	if (!str) return PyErr_NoMemory();
 	out = PyBytes_AS_STRING(str);
-#if MYSQL_VERSION_ID < 32321
-	len = mysql_escape_string(out, in, size);
-#else
 	check_server_init(NULL);
 
 	if (self && PyModule_Check((PyObject*)self))
 		self = NULL;
-	if (self && self->open)
+	if (self && self->open) {
+#if MYSQL_VERSION_ID >= 50707 && !defined(MARIADB_BASE_VERSION)
+		len = mysql_real_escape_string_quote(&(self->connection), out, in, size, '\'');
+#else
 		len = mysql_real_escape_string(&(self->connection), out, in, size);
-	else
-		len = mysql_escape_string(out, in, size);
 #endif
+	} else {
+		len = mysql_escape_string(out, in, size);
+	}
 	if (_PyBytes_Resize(&str, len) < 0) return NULL;
 	return (str);
 }
@@ -1148,15 +1116,16 @@ _mysql_string_literal(
 		return PyErr_NoMemory();
 	}
 	out = PyBytes_AS_STRING(str);
-#if MYSQL_VERSION_ID < 32321
-	len = mysql_escape_string(out+1, in, size);
-#else
 	check_server_init(NULL);
-	if (self && self->open)
+	if (self && self->open) {
+#if MYSQL_VERSION_ID >= 50707 && !defined(MARIADB_BASE_VERSION)
+		len = mysql_real_escape_string_quote(&(self->connection), out+1, in, size, '\'');
+#else
 		len = mysql_real_escape_string(&(self->connection), out+1, in, size);
-	else
-		len = mysql_escape_string(out+1, in, size);
 #endif
+	} else {
+		len = mysql_escape_string(out+1, in, size);
+	}
 	*out = *(out+len+1) = '\'';
 	if (_PyBytes_Resize(&str, len+2) < 0) return NULL;
 	Py_DECREF(s);
@@ -1300,12 +1269,11 @@ the Cursor.description attribute.\n\
 static PyObject *
 _mysql_ResultObject_describe(
 	_mysql_ResultObject *self,
-	PyObject *args)
+	PyObject *noargs)
 {
 	PyObject *d;
 	MYSQL_FIELD *fields;
 	unsigned int i, n;
-	if (!PyArg_ParseTuple(args, "")) return NULL;
 	check_result_connection(self);
 	n = mysql_num_fields(self->result);
 	fields = mysql_fetch_fields(self->result);
@@ -1336,12 +1304,11 @@ static char _mysql_ResultObject_field_flags__doc__[] =
 static PyObject *
 _mysql_ResultObject_field_flags(
 	_mysql_ResultObject *self,
-	PyObject *args)
+	PyObject *noargs)
 {
 	PyObject *d;
 	MYSQL_FIELD *fields;
 	unsigned int i, n;
-	if (!PyArg_ParseTuple(args, "")) return NULL;
 	check_result_connection(self);
 	n = mysql_num_fields(self->result);
 	fields = mysql_fetch_fields(self->result);
@@ -1620,8 +1587,6 @@ _mysql_ResultObject_fetch_row(
 	return NULL;
 }
 
-#if MYSQL_VERSION_ID >= 32303
-
 static char _mysql_ConnectionObject_change_user__doc__[] =
 "Changes the user and causes the database specified by db to\n\
 become the default (current) database on the connection\n\
@@ -1660,7 +1625,6 @@ _mysql_ConnectionObject_change_user(
 	Py_INCREF(Py_None);
 	return Py_None;
 }
-#endif
 
 static char _mysql_ConnectionObject_character_set_name__doc__[] =
 "Returns the default character set for the current connection.\n\
@@ -1670,20 +1634,14 @@ Non-standard.\n\
 static PyObject *
 _mysql_ConnectionObject_character_set_name(
 	_mysql_ConnectionObject *self,
-	PyObject *args)
+	PyObject *noargs)
 {
 	const char *s;
-	if (!PyArg_ParseTuple(args, "")) return NULL;
 	check_connection(self);
-#if MYSQL_VERSION_ID >= 32321
 	s = mysql_character_set_name(&(self->connection));
-#else
-	s = "latin1";
-#endif
 	return PyString_FromString(s);
 }
 
-#if MYSQL_VERSION_ID >= 50007
 static char _mysql_ConnectionObject_set_character_set__doc__[] =
 "Sets the default character set for the current connection.\n\
 Non-standard.\n\
@@ -1705,7 +1663,6 @@ _mysql_ConnectionObject_set_character_set(
 	Py_INCREF(Py_None);
 	return Py_None;
 }
-#endif
 
 #if MYSQL_VERSION_ID >= 50010
 static char _mysql_ConnectionObject_get_character_set_info__doc__[] =
@@ -1732,12 +1689,11 @@ Non-standard.\n\
 static PyObject *
 _mysql_ConnectionObject_get_character_set_info(
 	_mysql_ConnectionObject *self,
-	PyObject *args)
+	PyObject *noargs)
 {
 	PyObject *result;
 	MY_CHARSET_INFO cs;
 
-	if (!PyArg_ParseTuple(args, "")) return NULL;
 	check_connection(self);
 	mysql_get_character_set_info(&(self->connection), &cs);
 	if (!(result = PyDict_New())) return NULL;
@@ -1761,9 +1717,8 @@ the client library version.";
 static PyObject *
 _mysql_get_client_info(
 	PyObject *self,
-	PyObject *args)
+	PyObject *noargs)
 {
-	if (!PyArg_ParseTuple(args, "")) return NULL;
 	check_server_init(NULL);
 	return PyString_FromString(mysql_get_client_info());
 }
@@ -1776,9 +1731,8 @@ version. Non-standard.\n\
 static PyObject *
 _mysql_ConnectionObject_get_host_info(
 	_mysql_ConnectionObject *self,
-	PyObject *args)
+	PyObject *noargs)
 {
-	if (!PyArg_ParseTuple(args, "")) return NULL;
 	check_connection(self);
 	return PyString_FromString(mysql_get_host_info(&(self->connection)));
 }
@@ -1791,9 +1745,8 @@ used by the current connection. Non-standard.\n\
 static PyObject *
 _mysql_ConnectionObject_get_proto_info(
 	_mysql_ConnectionObject *self,
-	PyObject *args)
+	PyObject *noargs)
 {
-	if (!PyArg_ParseTuple(args, "")) return NULL;
 	check_connection(self);
 	return PyInt_FromLong((long)mysql_get_proto_info(&(self->connection)));
 }
@@ -1806,9 +1759,8 @@ Non-standard.\n\
 static PyObject *
 _mysql_ConnectionObject_get_server_info(
 	_mysql_ConnectionObject *self,
-	PyObject *args)
+	PyObject *noargs)
 {
-	if (!PyArg_ParseTuple(args, "")) return NULL;
 	check_connection(self);
 	return PyString_FromString(mysql_get_server_info(&(self->connection)));
 }
@@ -1822,10 +1774,9 @@ Cursor.messages.\n\
 static PyObject *
 _mysql_ConnectionObject_info(
 	_mysql_ConnectionObject *self,
-	PyObject *args)
+	PyObject *noargs)
 {
 	const char *s;
-	if (!PyArg_ParseTuple(args, "")) return NULL;
 	check_connection(self);
 	s = mysql_info(&(self->connection));
 	if (s) return PyString_FromString(s);
@@ -1857,10 +1808,9 @@ in the server.\n\
 static PyObject *
 _mysql_ConnectionObject_insert_id(
 	_mysql_ConnectionObject *self,
-	PyObject *args)
+	PyObject *noargs)
 {
 	my_ulonglong r;
-	if (!PyArg_ParseTuple(args, "")) return NULL;
 	check_connection(self);
 	Py_BEGIN_ALLOW_THREADS
 	r = mysql_insert_id(&(self->connection));
@@ -1898,15 +1848,10 @@ on most cursor classes. Use Cursor.rowcount.\n\
 static PyObject *
 _mysql_ConnectionObject_field_count(
 	_mysql_ConnectionObject *self,
-	PyObject *args)
+	PyObject *noargs)
 {
-	if (!PyArg_ParseTuple(args, "")) return NULL;
 	check_connection(self);
-#if MYSQL_VERSION_ID < 32224
-	return PyInt_FromLong((long)mysql_num_fields(&(self->connection)));
-#else
 	return PyInt_FromLong((long)mysql_field_count(&(self->connection)));
-#endif
 }	
 
 static char _mysql_ResultObject_num_fields__doc__[] =
@@ -1915,9 +1860,8 @@ static char _mysql_ResultObject_num_fields__doc__[] =
 static PyObject *
 _mysql_ResultObject_num_fields(
 	_mysql_ResultObject *self,
-	PyObject *args)
+	PyObject *noargs)
 {
-	if (!PyArg_ParseTuple(args, "")) return NULL;
 	check_result_connection(self);
 	return PyInt_FromLong((long)mysql_num_fields(self->result));
 }	
@@ -1931,9 +1875,8 @@ set has been read.\n\
 static PyObject *
 _mysql_ResultObject_num_rows(
 	_mysql_ResultObject *self,
-	PyObject *args)
+	PyObject *noargs)
 {
-	if (!PyArg_ParseTuple(args, "")) return NULL;
 	check_result_connection(self);
 	return PyLong_FromUnsignedLongLong(mysql_num_rows(self->result));
 }	
@@ -2028,7 +1971,8 @@ static char _mysql_ConnectionObject_read_query_result__doc__[] =
 
 static PyObject *
 _mysql_ConnectionObject_read_query_result(
-	_mysql_ConnectionObject *self)
+	_mysql_ConnectionObject *self,
+	PyObject *noargs)
 {
 	int r;
 	MYSQL *mysql = &(self->connection);
@@ -2079,17 +2023,12 @@ have shutdown privileges. Non-standard.\n\
 static PyObject *
 _mysql_ConnectionObject_shutdown(
 	_mysql_ConnectionObject *self,
-	PyObject *args)
+	PyObject *noargs)
 {
 	int r;
-	if (!PyArg_ParseTuple(args, "")) return NULL;
 	check_connection(self);
 	Py_BEGIN_ALLOW_THREADS
-	r = mysql_shutdown(&(self->connection)
-#if MYSQL_VERSION_ID >= 40103
-		, SHUTDOWN_DEFAULT
-#endif
-		);
+	r = mysql_shutdown(&(self->connection), SHUTDOWN_DEFAULT);
 	Py_END_ALLOW_THREADS
 	if (r) return _mysql_Exception(self);
 	Py_INCREF(Py_None);
@@ -2106,10 +2045,9 @@ questions, reloads, and open tables. Non-standard.\n\
 static PyObject *
 _mysql_ConnectionObject_stat(
 	_mysql_ConnectionObject *self,
-	PyObject *args)
+	PyObject *noargs)
 {
 	const char *s;
-	if (!PyArg_ParseTuple(args, "")) return NULL;
 	check_connection(self);
 	Py_BEGIN_ALLOW_THREADS
 	s = mysql_stat(&(self->connection));
@@ -2127,12 +2065,11 @@ None is returned. Non-standard.\n\
 static PyObject *
 _mysql_ConnectionObject_store_result(
 	_mysql_ConnectionObject *self,
-	PyObject *args)
+	PyObject *noargs)
 {
 	PyObject *arglist=NULL, *kwarglist=NULL, *result=NULL;
 	_mysql_ResultObject *r=NULL;
 
-	if (!PyArg_ParseTuple(args, "")) return NULL;
 	check_connection(self);
 	arglist = Py_BuildValue("(OiO)", self, 0, self->converter);
 	if (!arglist) goto error;
@@ -2168,10 +2105,9 @@ Non-standard.";
 static PyObject *
 _mysql_ConnectionObject_thread_id(
 	_mysql_ConnectionObject *self,
-	PyObject *args)
+	PyObject *noargs)
 {
 	unsigned long pid;
-	if (!PyArg_ParseTuple(args, "")) return NULL;
 	check_connection(self);
 	Py_BEGIN_ALLOW_THREADS
 	pid = mysql_thread_id(&(self->connection));
@@ -2188,12 +2124,11 @@ None is returned. Non-standard.\n\
 static PyObject *
 _mysql_ConnectionObject_use_result(
 	_mysql_ConnectionObject *self,
-	PyObject *args)
+	PyObject *noargs)
 {
 	PyObject *arglist=NULL, *kwarglist=NULL, *result=NULL;
 	_mysql_ResultObject *r=NULL;
 
-	if (!PyArg_ParseTuple(args, "")) return NULL;
 	check_connection(self);
 	arglist = Py_BuildValue("(OiO)", self, 1, self->converter);
 	if (!arglist) return NULL;
@@ -2284,10 +2219,9 @@ static char _mysql_ResultObject_row_tell__doc__[] =
 static PyObject *
 _mysql_ResultObject_row_tell(
 	_mysql_ResultObject *self,
-	PyObject *args)
+	PyObject *noargs)
 {
 	MYSQL_ROW_OFFSET r;
-	if (!PyArg_ParseTuple(args, "")) return NULL;
 	check_result_connection(self);
 	if (self->use) {
 		PyErr_SetString(_mysql_ProgrammingError,
@@ -2321,7 +2255,7 @@ static PyMethodDef _mysql_ConnectionObject_methods[] = {
 	{
 		"affected_rows",
 		(PyCFunction)_mysql_ConnectionObject_affected_rows,
-		METH_VARARGS,
+		METH_NOARGS,
 		_mysql_ConnectionObject_affected_rows__doc__
 	},
 	{
@@ -2339,22 +2273,21 @@ static PyMethodDef _mysql_ConnectionObject_methods[] = {
 	{
 		"commit",
 		(PyCFunction)_mysql_ConnectionObject_commit,
-		METH_VARARGS,
+		METH_NOARGS,
 		_mysql_ConnectionObject_commit__doc__
 	},
 	{
 		"rollback",
 		(PyCFunction)_mysql_ConnectionObject_rollback,
-		METH_VARARGS,
+		METH_NOARGS,
 		_mysql_ConnectionObject_rollback__doc__
 	},
 	{
 		"next_result",
 		(PyCFunction)_mysql_ConnectionObject_next_result,
-		METH_VARARGS,
+		METH_NOARGS,
 		_mysql_ConnectionObject_next_result__doc__
 	},
-#if MYSQL_VERSION_ID >= 40100
 	{
 		"set_server_option",
 		(PyCFunction)_mysql_ConnectionObject_set_server_option,
@@ -2364,50 +2297,45 @@ static PyMethodDef _mysql_ConnectionObject_methods[] = {
 	{
 		"sqlstate",
 		(PyCFunction)_mysql_ConnectionObject_sqlstate,
-		METH_VARARGS,
+		METH_NOARGS,
 		_mysql_ConnectionObject_sqlstate__doc__
 	},
 	{
 		"warning_count",
 		(PyCFunction)_mysql_ConnectionObject_warning_count,
-		METH_VARARGS,
+		METH_NOARGS,
 		_mysql_ConnectionObject_warning_count__doc__
 	},
-#endif
-#if MYSQL_VERSION_ID >= 32303
 	{
 		"change_user",
 		(PyCFunction)_mysql_ConnectionObject_change_user,
 		METH_VARARGS | METH_KEYWORDS,
 		_mysql_ConnectionObject_change_user__doc__
 	},
-#endif
 	{
 		"character_set_name",
 		(PyCFunction)_mysql_ConnectionObject_character_set_name,
-		METH_VARARGS,
+		METH_NOARGS,
 		_mysql_ConnectionObject_character_set_name__doc__
 	},
-#if MYSQL_VERSION_ID >= 50007
 	{
 		"set_character_set",
 		(PyCFunction)_mysql_ConnectionObject_set_character_set,
 		METH_VARARGS,
 		_mysql_ConnectionObject_set_character_set__doc__
 	},
-#endif
 #if MYSQL_VERSION_ID >= 50010
 	{
 		"get_character_set_info",
 		(PyCFunction)_mysql_ConnectionObject_get_character_set_info,
-		METH_VARARGS,
+		METH_NOARGS,
 		_mysql_ConnectionObject_get_character_set_info__doc__
 	},
 #endif
 	{
 		"close",
 		(PyCFunction)_mysql_ConnectionObject_close,
-		METH_VARARGS,
+		METH_NOARGS,
 		_mysql_ConnectionObject_close__doc__
 	},
 	{
@@ -2419,7 +2347,7 @@ static PyMethodDef _mysql_ConnectionObject_methods[] = {
 	{
 		"dump_debug_info",
 		(PyCFunction)_mysql_ConnectionObject_dump_debug_info,
-		METH_VARARGS,
+		METH_NOARGS,
 		_mysql_ConnectionObject_dump_debug_info__doc__
 	},
 	{
@@ -2437,49 +2365,49 @@ static PyMethodDef _mysql_ConnectionObject_methods[] = {
 	{
 		"error",
 		(PyCFunction)_mysql_ConnectionObject_error,
-		METH_VARARGS,
+		METH_NOARGS,
 		_mysql_ConnectionObject_error__doc__
 	},
 	{
 		"errno",
 		(PyCFunction)_mysql_ConnectionObject_errno,
-		METH_VARARGS,
+		METH_NOARGS,
 		_mysql_ConnectionObject_errno__doc__
 	},
 	{
 		"field_count",
 		(PyCFunction)_mysql_ConnectionObject_field_count,
-		METH_VARARGS,
+		METH_NOARGS,
 		_mysql_ConnectionObject_field_count__doc__
 	}, 
 	{
 		"get_host_info",
 		(PyCFunction)_mysql_ConnectionObject_get_host_info,
-		METH_VARARGS,
+		METH_NOARGS,
 		_mysql_ConnectionObject_get_host_info__doc__
 	},
 	{
 		"get_proto_info",
 		(PyCFunction)_mysql_ConnectionObject_get_proto_info,
-		METH_VARARGS,
+		METH_NOARGS,
 		_mysql_ConnectionObject_get_proto_info__doc__
 	},
 	{
 		"get_server_info",
 		(PyCFunction)_mysql_ConnectionObject_get_server_info,
-		METH_VARARGS,
+		METH_NOARGS,
 		_mysql_ConnectionObject_get_server_info__doc__
 	},
 	{
 		"info",
 		(PyCFunction)_mysql_ConnectionObject_info,
-		METH_VARARGS,
+		METH_NOARGS,
 		_mysql_ConnectionObject_info__doc__
 	},
 	{
 		"insert_id",
 		(PyCFunction)_mysql_ConnectionObject_insert_id,
-		METH_VARARGS,
+		METH_NOARGS,
 		_mysql_ConnectionObject_insert_id__doc__
 	},
 	{
@@ -2521,19 +2449,19 @@ static PyMethodDef _mysql_ConnectionObject_methods[] = {
 	{
 		"shutdown",
 		(PyCFunction)_mysql_ConnectionObject_shutdown,
-		METH_VARARGS,
+		METH_NOARGS,
 		_mysql_ConnectionObject_shutdown__doc__
 	},
 	{
 		"stat",
 		(PyCFunction)_mysql_ConnectionObject_stat,
-		METH_VARARGS,
+		METH_NOARGS,
 		_mysql_ConnectionObject_stat__doc__
 	},
 	{
 		"store_result",
 		(PyCFunction)_mysql_ConnectionObject_store_result,
-		METH_VARARGS,
+		METH_NOARGS,
 		_mysql_ConnectionObject_store_result__doc__
 	},
 	{
@@ -2544,13 +2472,13 @@ static PyMethodDef _mysql_ConnectionObject_methods[] = {
 	{
 		"thread_id",
 		(PyCFunction)_mysql_ConnectionObject_thread_id,
-		METH_VARARGS,
+		METH_NOARGS,
 		_mysql_ConnectionObject_thread_id__doc__
 	},
 	{
 		"use_result",
 		(PyCFunction)_mysql_ConnectionObject_use_result,
-		METH_VARARGS,
+		METH_NOARGS,
 		_mysql_ConnectionObject_use_result__doc__
 	},
 	{NULL,              NULL} /* sentinel */
@@ -2576,7 +2504,7 @@ static struct PyMemberDef _mysql_ConnectionObject_memberlist[] = {
 		T_UINT,
 		offsetof(_mysql_ConnectionObject,connection.server_capabilities),
 		READONLY,
-		"Capabilites of server; consult MySQLdb.constants.CLIENT"
+		"Capabilities of server; consult MySQLdb.constants.CLIENT"
 	},
 	{
 		"port",
@@ -2611,13 +2539,13 @@ static PyMethodDef _mysql_ResultObject_methods[] = {
 	{
 		"row_tell",
 		(PyCFunction)_mysql_ResultObject_row_tell,
-		METH_VARARGS,
+		METH_NOARGS,
 		_mysql_ResultObject_row_tell__doc__
 	},
 	{
 		"describe",
 		(PyCFunction)_mysql_ResultObject_describe,
-		METH_VARARGS,
+		METH_NOARGS,
 		_mysql_ResultObject_describe__doc__
 	},
 	{
@@ -2629,19 +2557,19 @@ static PyMethodDef _mysql_ResultObject_methods[] = {
 	{
 		"field_flags",
 		(PyCFunction)_mysql_ResultObject_field_flags,
-		METH_VARARGS,
+		METH_NOARGS,
 		_mysql_ResultObject_field_flags__doc__
 	},
 	{
 		"num_fields",
 		(PyCFunction)_mysql_ResultObject_num_fields,
-		METH_VARARGS,
+		METH_NOARGS,
 		_mysql_ResultObject_num_fields__doc__
 	},
 	{
 		"num_rows",
 		(PyCFunction)_mysql_ResultObject_num_rows,
-		METH_VARARGS,
+		METH_NOARGS,
 		_mysql_ResultObject_num_rows__doc__
 	},
 	{NULL,              NULL} /* sentinel */
@@ -2711,12 +2639,7 @@ _mysql_ResultObject_setattro(
 }
 
 PyTypeObject _mysql_ConnectionObject_Type = {
-#ifdef IS_PY3K
     PyVarObject_HEAD_INIT(NULL, 0)
-#else
-	PyObject_HEAD_INIT(NULL)
-	0,
-#endif
 	"_mysql.connection", /* (char *)tp_name For printing */
 	sizeof(_mysql_ConnectionObject),
 	0,
@@ -2783,12 +2706,7 @@ PyTypeObject _mysql_ConnectionObject_Type = {
 } ;
 
 PyTypeObject _mysql_ResultObject_Type = {
-#ifdef IS_PY3K
     PyVarObject_HEAD_INIT(NULL, 0)
-#else
-	PyObject_HEAD_INIT(NULL)
-	0,
-#endif
 	"_mysql.result",
 	sizeof(_mysql_ResultObject),
 	0,
@@ -2902,17 +2820,15 @@ _mysql_methods[] = {
 	{
 		"get_client_info",
 		(PyCFunction)_mysql_get_client_info,
-		METH_VARARGS,
+		METH_NOARGS,
 		_mysql_get_client_info__doc__
 	},
-#if MYSQL_VERSION_ID >= 32314
 	{
 		"thread_safe",
 		(PyCFunction)_mysql_thread_safe,
-		METH_VARARGS,
+		METH_NOARGS,
 		_mysql_thread_safe__doc__
 	},
-#endif
 	{
 		"server_init",
 		(PyCFunction)_mysql_server_init,
@@ -2935,13 +2851,11 @@ _mysql_NewException(
 	char *name)
 {
 	PyObject *e;
-
 	if (!(e = PyDict_GetItemString(edict, name)))
 		return NULL;
-	if (PyDict_SetItemString(dict, name, e)) return NULL;
-#ifdef PYPY_VERSION
+	if (PyDict_SetItemString(dict, name, e))
+		return NULL;
 	Py_INCREF(e);
-#endif
 	return e;
 }
 
