@@ -1,8 +1,13 @@
 #!/usr/bin/env python
+# -*- coding: utf-8 -*-
 import capabilities
 from datetime import timedelta
+from contextlib import closing
 import unittest
 import MySQLdb
+from MySQLdb.compat import unicode
+from MySQLdb import cursors
+from configdb import connection_factory
 import warnings
 
 
@@ -43,7 +48,7 @@ class test_MySQLdb(capabilities.DatabaseTest):
         c = self.cursor
         self.create_table(('pos INT', 'tree CHAR(20)'))
         c.executemany("INSERT INTO %s (pos,tree) VALUES (%%s,%%s)" % self.table,
-                      list(enumerate('ash birch cedar larch pine'.split())))
+                      list(enumerate('ash birch cedar Lärche pine'.split())))
         db.commit()
 
         c.execute("""
@@ -54,10 +59,10 @@ class test_MySQLdb(capabilities.DatabaseTest):
         """ % self.table)
         db.commit()
 
-        c.callproc('test_sp', ('larch',))
+        c.callproc('test_sp', ('Lärche',))
         rows = c.fetchall()
-        self.assertEquals(len(rows), 1)
-        self.assertEquals(rows[0][0], 3)
+        self.assertEqual(len(rows), 1)
+        self.assertEqual(rows[0][0], 3)
         c.nextset()
 
         c.execute("DROP PROCEDURE test_sp")
@@ -155,6 +160,45 @@ class test_MySQLdb(capabilities.DatabaseTest):
             return
         self.fail("Should raise ProgrammingError")
 
+    def test_warning_propagation(self):
+        with warnings.catch_warnings():
+            # Ignore all warnings other than MySQLdb generated ones
+            warnings.simplefilter("ignore")
+            warnings.simplefilter("error", category=MySQLdb.Warning)
+
+            # verify for both buffered and unbuffered cursor types
+            for cursor_class in (cursors.Cursor, cursors.SSCursor):
+                c = self.connection.cursor(cursor_class)
+                try:
+                    c.execute("SELECT CAST('124b' AS SIGNED)")
+                    c.fetchall()
+                except MySQLdb.Warning as e:
+                    # Warnings should have errorcode and string message, just like exceptions
+                    self.assertEqual(len(e.args), 2)
+                    self.assertEqual(e.args[0], 1292)
+                    self.assertTrue(isinstance(e.args[1], unicode))
+                else:
+                    self.fail("Should raise Warning")
+                finally:
+                    c.close()
+
+    def test_binary_prefix(self):
+        # verify prefix behaviour when enabled, disabled and for default (disabled)
+        for binary_prefix in (True, False, None):
+            kwargs = self.connect_kwargs.copy()
+            # needs to be set to can guarantee CHARSET response for normal strings
+            kwargs['charset'] = 'utf8'
+            if binary_prefix != None:
+                kwargs['binary_prefix'] = binary_prefix
+
+            with closing(connection_factory(**kwargs)) as conn:
+                with closing(conn.cursor()) as c:
+                    c.execute('SELECT CHARSET(%s)', (MySQLdb.Binary(b'raw bytes'),))
+                    self.assertEqual(c.fetchall()[0][0], 'binary' if binary_prefix else 'utf8')
+                    # normal strings should not get prefix
+                    c.execute('SELECT CHARSET(%s)', ('str',))
+                    self.assertEqual(c.fetchall()[0][0], 'utf8')
+
 
 if __name__ == '__main__':
     if test_MySQLdb.leak_test:
